@@ -6,7 +6,7 @@
  *
  ********************************************************************
  *
- * Copyright (C) 1994-1999 Jeff Tranter (tranter@pobox.com)
+ * Copyright (C) 1994-2001 Jeff Tranter (tranter@pobox.com)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -54,17 +54,19 @@
 #include <sys/ioctl.h>
 #include <sys/wait.h>
 #include <sys/mtio.h>
+#include <linux/types.h>
 #include <linux/cdrom.h>
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,1,0)
 #include <linux/ucdrom.h>
 #endif
 #include <linux/fd.h>
+#include <sys/mount.h>
 #include <scsi/scsi.h>
 #include <scsi/sg.h>
 #include <scsi/scsi_ioctl.h>
 
 /* Global Variables */
-const char *version = "2.0.2"; /* program version */
+const char *version = "2.0.3"; /* program version */
 int a_option = 0; /* command flags and arguments */
 int c_option = 0;
 int d_option = 0;
@@ -251,6 +253,8 @@ Given name, such as foo, see if any of the following exist:
 - foo (if foo starts with '.' or '/')
 - /dev/foo
 - /mnt/foo
+- /dev/cdroms/foo
+- /dev/cdroms/foo0
 - /dev/dsk/foo
 - /dev/rdsk/foo
 - /mnt/foo
@@ -280,7 +284,19 @@ static char *FindDevice(const char *name)
   if (FileExists(buf))
       return buf;
 
-  /* for systems that use devfs such as Solaris and probably Linux in future */
+  /* for devfs under Linux */
+  strcpy(buf, "/dev/cdroms/");
+  strcat(buf, name);
+  if (FileExists(buf))
+      return buf;
+
+  strcpy(buf, "/dev/cdroms/");
+  strcat(buf, name);
+  strcat(buf, "0");
+  if (FileExists(buf))
+      return buf;
+
+  /* for devfs under Solaris */
   strcpy(buf, "/dev/rdsk/");
   strcat(buf, name);
   if (FileExists(buf))
@@ -417,6 +433,11 @@ static int EjectScsi(int fd)
   scsi_cmd.cmd[4] = 2;
   scsi_cmd.cmd[5] = 0;
   status = ioctl(fd, SCSI_IOCTL_SEND_COMMAND, (void *)&scsi_cmd);
+  if (status != 0)
+    return 0;
+
+  /* force kernel to reread partition table when new disc inserted */
+  status = ioctl(fd, BLKRRPART);
   return (status == 0);
 }
 
@@ -546,7 +567,7 @@ static int MountableDevice(const char *name, char **mountName, char **deviceName
   
   while (fgets(line, sizeof(line), fp) != 0) {
     rc = sscanf(line, "%1023s %1023s", s1, s2);
-    if (rc >= 2 && strcmp(s2, name) == 0) {
+    if (rc >= 2 && s1[0] != '#' && strcmp(s2, name) == 0) {
       fclose(fp);
       *deviceName = strdup(s1);
       *mountName = strdup(s2);
@@ -601,6 +622,7 @@ static char *SymLink(const char *name)
   int status;
   char s1[1024];
   char s2[1024];
+  char s4[1024];
   char *s3;
 
   status = readlink(name, s1, sizeof(s1));
@@ -619,7 +641,8 @@ static char *SymLink(const char *name)
       strcat(s2, s1);
     }
   }
-  return strdup(s2);
+  realpath(s2, s4);
+  return strdup(s4);
 }
 
 
